@@ -24,7 +24,6 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <math.h>
-#include <ctype.h>
 #include <limits.h>
 #include <dirent.h>
 #include <fcntl.h>
@@ -37,6 +36,7 @@
 #include <paths.h>
 #include <assert.h>
 #include <values.h>
+#include <locale.h>
 
 #include <linux/kd.h>
 #include <linux/vt.h>
@@ -56,20 +56,9 @@
 #include "ply-utils.h"
 #include "ply-progress.h"
 
-#ifndef PLY_MAX_COMMAND_LINE_SIZE
-#define PLY_MAX_COMMAND_LINE_SIZE 4097
-#endif
-
 #define BOOT_DURATION_FILE     PLYMOUTH_TIME_DIRECTORY "/boot-duration"
 #define SHUTDOWN_DURATION_FILE PLYMOUTH_TIME_DIRECTORY "/shutdown-duration"
 
-typedef enum
-{
-        PLY_MODE_BOOT,
-        PLY_MODE_SHUTDOWN,
-        PLY_MODE_UPDATES
-} ply_mode_t;
-
 typedef struct
 {
         const char    *keys;
@@ -98,7 +87,7 @@ typedef struct
         ply_buffer_t           *entry_buffer;
         ply_list_t             *messages;
         ply_command_parser_t   *command_parser;
-        ply_mode_t              mode;
+        ply_boot_splash_mode_t  mode;
         ply_terminal_t         *local_console_terminal;
         ply_device_manager_t   *device_manager;
 
@@ -109,8 +98,6 @@ typedef struct
         double                  splash_delay;
         double                  device_timeout;
 
-        char                    kernel_command_line[PLY_MAX_COMMAND_LINE_SIZE];
-        uint32_t                kernel_command_line_is_set : 1;
         uint32_t                no_boot_log : 1;
         uint32_t                showing_details : 1;
         uint32_t                system_initialized : 1;
@@ -121,6 +108,7 @@ typedef struct
         uint32_t                is_inactive : 1;
         uint32_t                is_shown : 1;
         uint32_t                should_force_details : 1;
+        uint32_t                splash_is_becoming_idle : 1;
 
         char                   *override_splash_path;
         char                   *system_default_splash_path;
@@ -156,7 +144,7 @@ static void toggle_between_splash_and_details (state_t *state);
 static void tell_systemd_to_print_details (state_t *state);
 static void tell_systemd_to_stop_printing_details (state_t *state);
 #endif
-static const char *get_cache_file_for_mode (ply_mode_t mode);
+static const char *get_cache_file_for_mode (ply_boot_splash_mode_t mode);
 static void on_escape_pressed (state_t *state);
 static void on_enter (state_t    *state,
                       const char *line);
@@ -168,30 +156,8 @@ static void on_quit (state_t       *state,
                      bool           retain_splash,
                      ply_trigger_t *quit_trigger);
 static bool sh_is_init (state_t *state);
-
-static ply_boot_splash_mode_t
-get_splash_mode_from_mode (ply_mode_t mode)
-{
-        ply_boot_splash_mode_t splash_mode;
-        switch (mode) {
-                case PLY_MODE_BOOT:
-                        splash_mode = PLY_BOOT_SPLASH_MODE_BOOT_UP;
-                        break;
-                case PLY_MODE_SHUTDOWN:
-                        splash_mode = PLY_BOOT_SPLASH_MODE_SHUTDOWN;
-                        break;
-                case PLY_MODE_UPDATES:
-                        splash_mode = PLY_BOOT_SPLASH_MODE_UPDATES;
-                        break;
-                default:
-                        splash_mode = PLY_BOOT_SPLASH_MODE_INVALID;
-                        break;
-        }
-
-        assert (splash_mode != PLY_BOOT_SPLASH_MODE_INVALID);
-
-        return splash_mode;
-}
+static void cancel_pending_delayed_show (state_t *state);
+static void prepare_logging (state_t *state);
 
 static void
 on_session_output (state_t    *state,
@@ -226,26 +192,32 @@ static void
 on_change_mode (state_t    *state,
                 const char *mode)
 {
-        ply_boot_splash_mode_t splash_mode;
+        ply_trace ("updating mode to '%s'", mode);
+        if (strcmp (mode, "boot-up") == 0)
+                state->mode = PLY_BOOT_SPLASH_MODE_BOOT_UP;
+        else if (strcmp (mode, "shutdown") == 0)
+                state->mode = PLY_BOOT_SPLASH_MODE_SHUTDOWN;
+        else if (strcmp (mode, "reboot") == 0)
+                state->mode = PLY_BOOT_SPLASH_MODE_REBOOT;
+        else if (strcmp (mode, "updates") == 0)
+                state->mode = PLY_BOOT_SPLASH_MODE_UPDATES;
+        else if (strcmp (mode, "system-upgrade") == 0)
+                state->mode = PLY_BOOT_SPLASH_MODE_SYSTEM_UPGRADE;
+        else if (strcmp (mode, "firmware-upgrade") == 0)
+                state->mode = PLY_BOOT_SPLASH_MODE_FIRMWARE_UPGRADE;
+        else
+                return;
+
+        if (state->session != NULL) {
+                prepare_logging (state);
+        }
 
         if (state->boot_splash == NULL) {
                 ply_trace ("no splash set");
                 return;
         }
 
-        ply_trace ("updating mode to '%s'", mode);
-        if (strcmp (mode, "boot-up") == 0)
-                state->mode = PLY_MODE_BOOT;
-        else if (strcmp (mode, "shutdown") == 0)
-                state->mode = PLY_MODE_SHUTDOWN;
-        else if (strcmp (mode, "updates") == 0)
-                state->mode = PLY_MODE_UPDATES;
-        else
-                return;
-
-        splash_mode = get_splash_mode_from_mode (state->mode);
-
-        if (!ply_boot_splash_show (state->boot_splash, splash_mode)) {
+        if (!ply_boot_splash_show (state->boot_splash, state->mode)) {
                 ply_trace ("failed to update splash");
                 return;
         }
@@ -295,8 +267,8 @@ load_settings (state_t    *state,
 {
         ply_key_file_t *key_file = NULL;
         bool settings_loaded = false;
-        const char *scale_string;
-        const char *splash_string;
+        char *scale_string = NULL;
+        char *splash_string = NULL;
 
         ply_trace ("Trying to load %s", path);
         key_file = ply_key_file_new (path);
@@ -306,40 +278,42 @@ load_settings (state_t    *state,
 
         splash_string = ply_key_file_get_value (key_file, "Daemon", "Theme");
 
-        if (splash_string == NULL)
-                goto out;
-
-        asprintf (theme_path,
-                  PLYMOUTH_RUNTIME_THEME_PATH "%s/%s.plymouth",
-                  splash_string, splash_string);
-        ply_trace ("Checking if %s exists", *theme_path);
-        if (!ply_file_exists (*theme_path)) {
-                ply_trace ("%s not found, fallbacking to " PLYMOUTH_THEME_PATH,
-                           *theme_path);
+        if (splash_string != NULL) {
                 asprintf (theme_path,
-                          PLYMOUTH_THEME_PATH "%s/%s.plymouth",
+                          PLYMOUTH_RUNTIME_THEME_PATH "%s/%s.plymouth",
                           splash_string, splash_string);
+                ply_trace ("Checking if %s exists", *theme_path);
+                if (!ply_file_exists (*theme_path)) {
+                        ply_trace ("%s not found, fallbacking to " PLYMOUTH_THEME_PATH,
+                                   *theme_path);
+                        asprintf (theme_path,
+                                  PLYMOUTH_THEME_PATH "%s/%s.plymouth",
+                                  splash_string, splash_string);
+                }
         }
 
         if (isnan (state->splash_delay)) {
-                const char *delay_string;
+                char *delay_string;
 
                 delay_string = ply_key_file_get_value (key_file, "Daemon", "ShowDelay");
 
                 if (delay_string != NULL) {
                         state->splash_delay = atof (delay_string);
                         ply_trace ("Splash delay is set to %lf", state->splash_delay);
+                        free (delay_string);
                 }
         }
 
         if (isnan (state->device_timeout)) {
-                const char *timeout_string;
+                char *timeout_string;
 
                 timeout_string = ply_key_file_get_value (key_file, "Daemon", "DeviceTimeout");
 
                 if (timeout_string != NULL) {
                         state->device_timeout = atof (timeout_string);
                         ply_trace ("Device timeout is set to %lf", state->device_timeout);
+
+                        free (timeout_string);
                 }
         }
 
@@ -347,10 +321,12 @@ load_settings (state_t    *state,
 
         if (scale_string != NULL) {
                 ply_set_device_scale (strtoul (scale_string, NULL, 0));
+                free (scale_string);
         }
 
         settings_loaded = true;
 out:
+        free (splash_string);
         ply_key_file_free (key_file);
 
         return settings_loaded;
@@ -361,6 +337,8 @@ show_detailed_splash (state_t *state)
 {
         ply_boot_splash_t *splash;
 
+        cancel_pending_delayed_show (state);
+
         if (state->boot_splash != NULL)
                 return;
 
@@ -378,78 +356,38 @@ show_detailed_splash (state_t *state)
         update_display (state);
 }
 
-static const char *
-command_line_get_string_after_prefix (const char *command_line,
-                                      const char *prefix)
-{
-        char *argument;
-
-        argument = strstr (command_line, prefix);
-
-        if (argument == NULL)
-                return NULL;
-
-        if (argument == command_line ||
-            argument[-1] == ' ')
-                return argument + strlen (prefix);
-
-        return NULL;
-}
-
-static bool
-command_line_has_argument (const char *command_line,
-                           const char *argument)
-{
-        const char *string;
-
-        string = command_line_get_string_after_prefix (command_line, argument);
-
-        if (string == NULL)
-                return false;
-
-        if (!isspace ((int) string[0]) && string[0] != '\0')
-                return false;
-
-        return true;
-}
-
 static void
 find_override_splash (state_t *state)
 {
-        const char *splash_string;
+        char *splash_string;
 
         if (state->override_splash_path != NULL)
                 return;
 
-        splash_string = command_line_get_string_after_prefix (state->kernel_command_line,
-                                                              "plymouth.splash=");
+        splash_string = ply_kernel_command_line_get_key_value ("plymouth.splash=");
 
         if (splash_string != NULL) {
-                const char *end;
-                int length;
-
-                end = splash_string + strcspn (splash_string, " \n");
-                length = end - splash_string;
-
-                ply_trace ("Splash is configured to be '%*.*s'", length, length, splash_string);
-
+                ply_trace ("Splash is configured to be '%s'", splash_string);
                 asprintf (&state->override_splash_path,
-                          PLYMOUTH_RUNTIME_THEME_PATH "%*.*s/%*.*s.plymouth",
-                          length, length, splash_string, length, length, splash_string);
+                          PLYMOUTH_RUNTIME_THEME_PATH "%s/%s.plymouth",
+                          splash_string, splash_string);
+
                 ply_trace ("Checking if %s exists", state->override_splash_path);
                 if (!ply_file_exists (state->override_splash_path)) {
                         ply_trace ("%s not found, fallbacking to " PLYMOUTH_THEME_PATH,
                                    state->override_splash_path);
+                        free (state->override_splash_path);
                         asprintf (&state->override_splash_path,
-                                  PLYMOUTH_THEME_PATH "%*.*s/%*.*s.plymouth",
-                                  length, length, splash_string, length, length, splash_string);
+                                  PLYMOUTH_THEME_PATH "%s/%s.plymouth",
+                                  splash_string, splash_string);
                 }
+                free (splash_string);
         }
 
         if (isnan (state->splash_delay)) {
                 const char *delay_string;
 
-                delay_string = command_line_get_string_after_prefix (state->kernel_command_line, "plymouth.splash-delay=");
+                delay_string = ply_kernel_command_line_get_string_after_prefix ("plymouth.splash-delay=");
 
                 if (delay_string != NULL)
                         state->splash_delay = atof (delay_string);
@@ -461,7 +399,7 @@ find_force_scale (state_t *state)
 {
         const char *scale_string;
 
-        scale_string = command_line_get_string_after_prefix (state->kernel_command_line, "plymouth.force-scale=");
+        scale_string = ply_kernel_command_line_get_string_after_prefix ("plymouth.force-scale=");
 
         if (scale_string != NULL)
                 ply_set_device_scale (strtoul (scale_string, NULL, 0));
@@ -718,26 +656,32 @@ on_newroot (state_t    *state,
         chdir (root_dir);
         chroot (".");
         chdir ("/");
+        /* Update local now that we have /usr/share/locale available */
+        setlocale(LC_ALL, "");
         ply_progress_load_cache (state->progress, get_cache_file_for_mode (state->mode));
         if (state->boot_splash != NULL)
                 ply_boot_splash_root_mounted (state->boot_splash);
 }
 
 static const char *
-get_cache_file_for_mode (ply_mode_t mode)
+get_cache_file_for_mode (ply_boot_splash_mode_t mode)
 {
         const char *filename;
 
-        switch ((int) mode) {
-        case PLY_MODE_BOOT:
+        switch (mode) {
+        case PLY_BOOT_SPLASH_MODE_BOOT_UP:
                 filename = BOOT_DURATION_FILE;
                 break;
-        case PLY_MODE_SHUTDOWN:
+        case PLY_BOOT_SPLASH_MODE_SHUTDOWN:
+        case PLY_BOOT_SPLASH_MODE_REBOOT:
                 filename = SHUTDOWN_DURATION_FILE;
                 break;
-        case PLY_MODE_UPDATES:
+        case PLY_BOOT_SPLASH_MODE_UPDATES:
+        case PLY_BOOT_SPLASH_MODE_SYSTEM_UPGRADE:
+        case PLY_BOOT_SPLASH_MODE_FIRMWARE_UPGRADE:
                 filename = NULL;
                 break;
+        case PLY_BOOT_SPLASH_MODE_INVALID:
         default:
                 ply_error ("Unhandled case in %s line %d\n", __FILE__, __LINE__);
                 abort ();
@@ -753,17 +697,21 @@ get_log_file_for_state (state_t *state)
 {
         const char *filename;
 
-        switch ((int) state->mode) {
-        case PLY_MODE_BOOT:
+        switch (state->mode) {
+        case PLY_BOOT_SPLASH_MODE_BOOT_UP:
                 if (state->no_boot_log)
                         filename = NULL;
                 else
                         filename = PLYMOUTH_LOG_DIRECTORY "/boot.log";
                 break;
-        case PLY_MODE_SHUTDOWN:
-        case PLY_MODE_UPDATES:
+        case PLY_BOOT_SPLASH_MODE_SHUTDOWN:
+        case PLY_BOOT_SPLASH_MODE_REBOOT:
+        case PLY_BOOT_SPLASH_MODE_UPDATES:
+        case PLY_BOOT_SPLASH_MODE_SYSTEM_UPGRADE:
+        case PLY_BOOT_SPLASH_MODE_FIRMWARE_UPGRADE:
                 filename = _PATH_DEVNULL;
                 break;
+        case PLY_BOOT_SPLASH_MODE_INVALID:
         default:
                 ply_error ("Unhandled case in %s line %d\n", __FILE__, __LINE__);
                 abort ();
@@ -775,18 +723,22 @@ get_log_file_for_state (state_t *state)
 }
 
 static const char *
-get_log_spool_file_for_mode (ply_mode_t mode)
+get_log_spool_file_for_mode (ply_boot_splash_mode_t mode)
 {
         const char *filename;
 
-        switch ((int) mode) {
-        case PLY_MODE_BOOT:
+        switch (mode) {
+        case PLY_BOOT_SPLASH_MODE_BOOT_UP:
                 filename = PLYMOUTH_SPOOL_DIRECTORY "/boot.log";
                 break;
-        case PLY_MODE_SHUTDOWN:
-        case PLY_MODE_UPDATES:
+        case PLY_BOOT_SPLASH_MODE_SHUTDOWN:
+        case PLY_BOOT_SPLASH_MODE_REBOOT:
+        case PLY_BOOT_SPLASH_MODE_UPDATES:
+        case PLY_BOOT_SPLASH_MODE_SYSTEM_UPGRADE:
+        case PLY_BOOT_SPLASH_MODE_FIRMWARE_UPGRADE:
                 filename = NULL;
                 break;
+        case PLY_BOOT_SPLASH_MODE_INVALID:
         default:
                 ply_error ("Unhandled case in %s line %d\n", __FILE__, __LINE__);
                 abort ();
@@ -830,6 +782,8 @@ prepare_logging (state_t *state)
                 return;
         }
 
+        ply_terminal_session_close_log (state->session);
+
         logfile = get_log_file_for_state (state);
         if (logfile != NULL) {
                 bool log_opened;
@@ -851,6 +805,11 @@ on_system_initialized (state_t *state)
         ply_trace ("system now initialized, opening log");
         state->system_initialized = true;
 
+#ifdef PLY_ENABLE_SYSTEMD_INTEGRATION
+        if (state->is_attached)
+                tell_systemd_to_print_details (state);
+#endif
+
         prepare_logging (state);
 }
 
@@ -871,10 +830,10 @@ static bool
 plymouth_should_ignore_show_splash_calls (state_t *state)
 {
         ply_trace ("checking if plymouth should be running");
-        if (state->mode != PLY_MODE_BOOT || command_line_has_argument (state->kernel_command_line, "plymouth.force-splash"))
+        if (state->mode != PLY_BOOT_SPLASH_MODE_BOOT_UP || ply_kernel_command_line_has_argument ("plymouth.force-splash"))
                 return false;
 
-        if (command_line_has_argument (state->kernel_command_line, "plymouth.ignore-show-splash"))
+        if (ply_kernel_command_line_has_argument ("plymouth.ignore-show-splash"))
                 return true;
 
         return false;
@@ -883,18 +842,20 @@ plymouth_should_ignore_show_splash_calls (state_t *state)
 static bool
 sh_is_init (state_t *state)
 {
-        const char *init_string;
+        char *init_string = ply_kernel_command_line_get_key_value ("init=");
+        bool result = false;
         size_t length;
 
-        init_string = command_line_get_string_after_prefix (state->kernel_command_line, "init=");
-
         if (init_string) {
-                length = strcspn (init_string, " \n");
-                if (length > 2 && ply_string_has_prefix (init_string + length - 2, "sh"))
-                        return true;
+                length = strlen (init_string);
+                if (length > 2 && init_string[length - 2] == 's' &&
+                                  init_string[length - 1] == 'h')
+                        result = true;
+
+                free (init_string);
         }
 
-        return false;
+        return result;
 }
 
 static bool
@@ -902,7 +863,7 @@ plymouth_should_show_default_splash (state_t *state)
 {
         ply_trace ("checking if plymouth should show default splash");
 
-        const char const *strings[] = {
+        const char * const strings[] = {
                 "single", "1", "s", "S", "-S", NULL
         };
         int i;
@@ -911,28 +872,28 @@ plymouth_should_show_default_splash (state_t *state)
                 return false;
 
         for (i = 0; strings[i] != NULL; i++) {
-                if (command_line_has_argument (state->kernel_command_line, strings[i])) {
+                if (ply_kernel_command_line_has_argument (strings[i])) {
                         ply_trace ("no default splash because kernel command line has option \"%s\"", strings[i]);
                         return false;
                 }
         }
 
-        if (command_line_has_argument (state->kernel_command_line, "splash=verbose")) {
+        if (ply_kernel_command_line_has_argument ("splash=verbose")) {
                 ply_trace ("no default splash because kernel command line has option \"splash=verbose\"");
                 return false;
         }
 
-        if (command_line_has_argument (state->kernel_command_line, "rhgb")) {
+        if (ply_kernel_command_line_has_argument ("rhgb")) {
                 ply_trace ("using default splash because kernel command line has option \"rhgb\"");
                 return true;
         }
 
-        if (command_line_has_argument (state->kernel_command_line, "splash")) {
+        if (ply_kernel_command_line_has_argument ("splash")) {
                 ply_trace ("using default splash because kernel command line has option \"splash\"");
                 return true;
         }
 
-        if (command_line_has_argument (state->kernel_command_line, "splash=silent")) {
+        if (ply_kernel_command_line_has_argument ("splash=silent")) {
                 ply_trace ("using default splash because kernel command line has option \"splash=silent\"");
                 return true;
         }
@@ -1001,6 +962,8 @@ show_splash (state_t *state)
                                                           (ply_event_loop_timeout_handler_t)
                                                           show_splash,
                                                           state);
+                        /* Listen for ESC to show details */
+                        ply_device_manager_activate_keyboards (state->device_manager);
                         return;
                 }
         }
@@ -1151,7 +1114,7 @@ load_devices (state_t                   *state,
 static void
 quit_splash (state_t *state)
 {
-        ply_trace ("quiting splash");
+        ply_trace ("quitting splash");
         if (state->boot_splash != NULL) {
                 ply_trace ("freeing splash");
                 ply_boot_splash_free (state->boot_splash);
@@ -1174,6 +1137,9 @@ quit_splash (state_t *state)
 static void
 hide_splash (state_t *state)
 {
+        if (state->boot_splash && ply_boot_splash_uses_pixel_displays (state->boot_splash))
+                ply_device_manager_deactivate_renderers (state->device_manager);
+
         state->is_shown = false;
 
         cancel_pending_delayed_show (state);
@@ -1193,7 +1159,6 @@ dump_details_and_quit_splash (state_t *state)
         state->showing_details = false;
         toggle_between_splash_and_details (state);
 
-        ply_device_manager_deactivate_renderers (state->device_manager);
         hide_splash (state);
         quit_splash (state);
 }
@@ -1211,17 +1176,6 @@ on_hide_splash (state_t *state)
         dump_details_and_quit_splash (state);
 }
 
-#ifdef PLY_ENABLE_DEPRECATED_GDM_TRANSITION
-static void
-tell_gdm_to_transition (void)
-{
-        int fd;
-
-        fd = creat ("/var/spool/gdm/force-display-on-active-vt", 0644);
-        close (fd);
-}
-#endif
-
 static void
 quit_program (state_t *state)
 {
@@ -1237,13 +1191,6 @@ quit_program (state_t *state)
                 pid_file = NULL;
         }
 
-#ifdef PLY_ENABLE_DEPRECATED_GDM_TRANSITION
-        if (state->should_retain_splash &&
-            state->mode == PLY_MODE_BOOT)
-                tell_gdm_to_transition ();
-
-#endif
-
         if (state->deactivate_trigger != NULL) {
                 ply_trigger_pull (state->deactivate_trigger, NULL);
                 state->deactivate_trigger = NULL;
@@ -1255,12 +1202,8 @@ quit_program (state_t *state)
 }
 
 static void
-deactivate_splash (state_t *state)
+deactivate_console (state_t *state)
 {
-        assert (!state->is_inactive);
-
-        ply_device_manager_deactivate_renderers (state->device_manager);
-
         detach_from_running_session (state);
 
         if (state->local_console_terminal != NULL) {
@@ -1271,9 +1214,21 @@ deactivate_splash (state_t *state)
         }
 
         /* do not let any tty opened where we could write after deactivate */
-        if (command_line_has_argument (state->kernel_command_line, "plymouth.debug"))
+        if (ply_kernel_command_line_has_argument ("plymouth.debug"))
                 ply_logger_close_file (ply_logger_get_error_default ());
 
+}
+
+static void
+deactivate_splash (state_t *state)
+{
+        assert (!state->is_inactive);
+
+        if (state->boot_splash && ply_boot_splash_uses_pixel_displays (state->boot_splash))
+                ply_device_manager_deactivate_renderers (state->device_manager);
+
+        deactivate_console (state);
+
         state->is_inactive = true;
 
         ply_trigger_pull (state->deactivate_trigger, NULL);
@@ -1291,7 +1246,6 @@ on_boot_splash_idle (state_t *state)
         if (state->quit_trigger != NULL) {
                 if (!state->should_retain_splash) {
                         ply_trace ("hiding splash");
-                        ply_device_manager_deactivate_renderers (state->device_manager);
                         hide_splash (state);
                 }
 
@@ -1303,6 +1257,8 @@ on_boot_splash_idle (state_t *state)
                 ply_trace ("deactivating splash");
                 deactivate_splash (state);
         }
+
+        state->splash_is_becoming_idle = false;
 }
 
 static void
@@ -1310,6 +1266,7 @@ on_deactivate (state_t       *state,
                ply_trigger_t *deactivate_trigger)
 {
         if (state->is_inactive) {
+                deactivate_console (state);
                 ply_trigger_pull (deactivate_trigger, NULL);
                 return;
         }
@@ -1327,13 +1284,17 @@ on_deactivate (state_t       *state,
         ply_trace ("deactivating");
         cancel_pending_delayed_show (state);
 
+        ply_device_manager_pause (state->device_manager);
         ply_device_manager_deactivate_keyboards (state->device_manager);
 
         if (state->boot_splash != NULL) {
-                ply_boot_splash_become_idle (state->boot_splash,
-                                             (ply_boot_splash_on_idle_handler_t)
-                                             on_boot_splash_idle,
-                                             state);
+                if (!state->splash_is_becoming_idle) {
+                        ply_boot_splash_become_idle (state->boot_splash,
+                                                     (ply_boot_splash_on_idle_handler_t)
+                                                     on_boot_splash_idle,
+                                                     state);
+                        state->splash_is_becoming_idle = true;
+                }
         } else {
                 ply_trace ("deactivating splash");
                 deactivate_splash (state);
@@ -1359,7 +1320,10 @@ on_reactivate (state_t *state)
         }
 
         ply_device_manager_activate_keyboards (state->device_manager);
-        ply_device_manager_activate_renderers (state->device_manager);
+        if (state->boot_splash && ply_boot_splash_uses_pixel_displays (state->boot_splash))
+                ply_device_manager_activate_renderers (state->device_manager);
+
+        ply_device_manager_unpause (state->device_manager);
 
         state->is_inactive = false;
 
@@ -1410,10 +1374,13 @@ on_quit (state_t       *state,
                 dump_details_and_quit_splash (state);
                 quit_program (state);
         } else if (state->boot_splash != NULL) {
-                ply_boot_splash_become_idle (state->boot_splash,
-                                             (ply_boot_splash_on_idle_handler_t)
-                                             on_boot_splash_idle,
-                                             state);
+                if (!state->splash_is_becoming_idle) {
+                        ply_boot_splash_become_idle (state->boot_splash,
+                                                     (ply_boot_splash_on_idle_handler_t)
+                                                     on_boot_splash_idle,
+                                                     state);
+                        state->splash_is_becoming_idle = true;
+                }
         } else {
                 quit_program (state);
         }
@@ -1570,6 +1537,8 @@ on_backspace (state_t *state)
 
         bytes = ply_buffer_get_bytes (state->entry_buffer);
         size = ply_buffer_get_size (state->entry_buffer);
+        if (size == 0)
+                return;
 
         bytes_to_remove = MIN (size, PLY_UTF8_CHARACTER_SIZE_MAX);
         while ((previous_character_size = ply_utf8_character_get_size (bytes + size - bytes_to_remove, bytes_to_remove)) < bytes_to_remove) {
@@ -1749,7 +1718,6 @@ static ply_boot_splash_t *
 show_theme (state_t    *state,
             const char *theme_path)
 {
-        ply_boot_splash_mode_t splash_mode;
         ply_boot_splash_t *splash;
 
         if (theme_path != NULL)
@@ -1761,22 +1729,16 @@ show_theme (state_t    *state,
                 return NULL;
 
         attach_splash_to_devices (state, splash);
-        ply_device_manager_activate_renderers (state->device_manager);
+        if (ply_boot_splash_uses_pixel_displays (splash))
+                ply_device_manager_activate_renderers (state->device_manager);
 
-        splash_mode = get_splash_mode_from_mode (state->mode);
-
-        if (!ply_boot_splash_show (splash, splash_mode)) {
+        if (!ply_boot_splash_show (splash, state->mode)) {
                 ply_save_errno ();
                 ply_boot_splash_free (splash);
                 ply_restore_errno ();
                 return NULL;
         }
 
-#ifdef PLY_ENABLE_SYSTEMD_INTEGRATION
-        if (state->is_attached)
-                tell_systemd_to_print_details (state);
-#endif
-
         ply_device_manager_activate_keyboards (state->device_manager);
 
         return splash;
@@ -1823,6 +1785,10 @@ attach_to_running_session (state_t *state)
                 return false;
         }
 
+#ifdef PLY_ENABLE_SYSTEMD_INTEGRATION
+        tell_systemd_to_print_details (state);
+#endif
+
         state->is_redirected = should_be_redirected;
         state->is_attached = true;
         state->session = session;
@@ -1839,106 +1805,47 @@ detach_from_running_session (state_t *state)
         if (!state->is_attached)
                 return;
 
+#ifdef PLY_ENABLE_SYSTEMD_INTEGRATION
+        tell_systemd_to_stop_printing_details (state);
+#endif
+
         ply_trace ("detaching from terminal session");
         ply_terminal_session_detach (state->session);
         state->is_redirected = false;
         state->is_attached = false;
 }
 
-static bool
-get_kernel_command_line (state_t *state)
-{
-        int fd;
-        const char *remaining_command_line;
-        char *key;
-
-        if (state->kernel_command_line_is_set)
-                return true;
-
-        ply_trace ("opening /proc/cmdline");
-        fd = open ("/proc/cmdline", O_RDONLY);
-
-        if (fd < 0) {
-                ply_trace ("couldn't open it: %m");
-                return false;
-        }
-
-        ply_trace ("reading kernel command line");
-        if (read (fd, state->kernel_command_line, sizeof(state->kernel_command_line) - 1) < 0) {
-                ply_trace ("couldn't read it: %m");
-                close (fd);
-                return false;
-        }
-
-
-        /* we now use plymouth.argument for kernel commandline arguments.
-         * It used to be plymouth:argument. This bit just rewrites all : to be .
-         */
-        remaining_command_line = state->kernel_command_line;
-        while ((key = strstr (remaining_command_line, "plymouth:")) != NULL) {
-                char *colon;
-
-                colon = key + strlen ("plymouth");
-                *colon = '.';
-
-                remaining_command_line = colon + 1;
-        }
-        ply_trace ("Kernel command line is: '%s'", state->kernel_command_line);
-
-        close (fd);
-
-        state->kernel_command_line_is_set = true;
-        return true;
-}
-
 static void
 check_verbosity (state_t *state)
 {
-        const char *stream;
-        const char *path;
+        char *stream;
 
         ply_trace ("checking if tracing should be enabled");
 
-        stream = command_line_get_string_after_prefix (state->kernel_command_line,
-                                                       "plymouth.debug=stream:");
+        if (!debug_buffer_path)
+                debug_buffer_path = ply_kernel_command_line_get_key_value ("plymouth.debug=file:");
 
-        path = command_line_get_string_after_prefix (state->kernel_command_line,
-                                                     "plymouth.debug=file:");
-        if (stream != NULL || path != NULL ||
-            command_line_has_argument (state->kernel_command_line, "plymouth.debug")) {
+        stream = ply_kernel_command_line_get_key_value ("plymouth.debug=stream:");
+        if (stream != NULL || debug_buffer_path != NULL ||
+            ply_kernel_command_line_has_argument ("plymouth.debug")) {
                 int fd;
 
                 ply_trace ("tracing should be enabled!");
                 if (!ply_is_tracing ())
                         ply_toggle_tracing ();
 
-                if (path != NULL && debug_buffer_path == NULL) {
-                        char *end;
-
-                        debug_buffer_path = strdup (path);
-                        end = debug_buffer_path + strcspn (debug_buffer_path, " \n");
-                        *end = '\0';
-                }
-
                 if (debug_buffer == NULL)
                         debug_buffer = ply_buffer_new ();
 
                 if (stream != NULL) {
-                        char *end;
-                        char *stream_copy;
-
-                        stream_copy = strdup (stream);
-                        end = stream_copy + strcspn (stream_copy, " \n");
-                        *end = '\0';
-
-                        ply_trace ("streaming debug output to %s instead of screen", stream_copy);
-                        fd = open (stream_copy, O_RDWR | O_NOCTTY | O_CREAT, 0600);
+                        ply_trace ("streaming debug output to %s instead of screen", stream);
+                        fd = open (stream, O_RDWR | O_NOCTTY | O_CREAT, 0600);
 
                         if (fd < 0)
-                                ply_trace ("could not stream output to %s: %m", stream_copy);
+                                ply_trace ("could not stream output to %s: %m", stream);
                         else
                                 ply_logger_set_output_fd (ply_logger_get_error_default (), fd);
-                        free (stream_copy);
+                        free (stream);
                 } else {
                         const char *device;
                         char *file;
@@ -1983,7 +1890,7 @@ check_logging (state_t *state)
 
         ply_trace ("checking if console messages should be redirected and logged");
 
-        kernel_no_log = command_line_has_argument (state->kernel_command_line, "plymouth.nolog");
+        kernel_no_log = ply_kernel_command_line_has_argument ("plymouth.nolog");
         if (kernel_no_log)
                 state->no_boot_log = true;
 
@@ -2037,14 +1944,12 @@ initialize_environment (state_t *state)
 {
         ply_trace ("initializing minimal work environment");
 
-        if (!get_kernel_command_line (state))
-                return false;
-
         if (!state->default_tty)
                 if (getenv ("DISPLAY") != NULL && access (PLYMOUTH_PLUGIN_PATH "renderers/x11.so", F_OK) == 0)
                         state->default_tty = "/dev/tty";
         if (!state->default_tty) {
-                if (state->mode == PLY_MODE_SHUTDOWN)
+                if (state->mode == PLY_BOOT_SPLASH_MODE_SHUTDOWN ||
+                    state->mode == PLY_BOOT_SPLASH_MODE_REBOOT)
                         state->default_tty = SHUTDOWN_TTY;
                 else
                         state->default_tty = BOOT_TTY;
@@ -2240,11 +2145,17 @@ main (int    argc,
 
         if (mode_string != NULL) {
                 if (strcmp (mode_string, "shutdown") == 0)
-                        state.mode = PLY_MODE_SHUTDOWN;
+                        state.mode = PLY_BOOT_SPLASH_MODE_SHUTDOWN;
+                else if (strcmp (mode_string, "reboot") == 0)
+                        state.mode = PLY_BOOT_SPLASH_MODE_REBOOT;
                 else if (strcmp (mode_string, "updates") == 0)
-                        state.mode = PLY_MODE_UPDATES;
+                        state.mode = PLY_BOOT_SPLASH_MODE_UPDATES;
+                else if (strcmp (mode_string, "system-upgrade") == 0)
+                        state.mode = PLY_BOOT_SPLASH_MODE_SYSTEM_UPGRADE;
+                else if (strcmp (mode_string, "firmware-upgrade") == 0)
+                        state.mode = PLY_BOOT_SPLASH_MODE_FIRMWARE_UPGRADE;
                 else
-                        state.mode = PLY_MODE_BOOT;
+                        state.mode = PLY_BOOT_SPLASH_MODE_BOOT_UP;
 
                 free (mode_string);
         }
@@ -2252,11 +2163,8 @@ main (int    argc,
         if (tty != NULL)
                 state.default_tty = tty;
 
-        if (kernel_command_line != NULL) {
-                strncpy (state.kernel_command_line, kernel_command_line, sizeof(state.kernel_command_line));
-                state.kernel_command_line[sizeof(state.kernel_command_line) - 1] = '\0';
-                state.kernel_command_line_is_set = true;
-        }
+        if (kernel_command_line != NULL)
+                ply_kernel_command_line_override (kernel_command_line);
 
         if (geteuid () != 0) {
                 ply_error ("plymouthd must be run as root user");
@@ -2348,15 +2256,17 @@ main (int    argc,
         find_system_default_splash (&state);
         find_distribution_default_splash (&state);
 
-        if (command_line_has_argument (state.kernel_command_line, "plymouth.ignore-serial-consoles"))
+        if (ply_kernel_command_line_has_argument ("plymouth.ignore-serial-consoles"))
                 device_manager_flags |= PLY_DEVICE_MANAGER_FLAGS_IGNORE_SERIAL_CONSOLES;
 
-        if (command_line_has_argument (state.kernel_command_line, "plymouth.ignore-udev") ||
+        if (ply_kernel_command_line_has_argument ("plymouth.ignore-udev") ||
             (getenv ("DISPLAY") != NULL))
                 device_manager_flags |= PLY_DEVICE_MANAGER_FLAGS_IGNORE_UDEV;
 
         if (!plymouth_should_show_default_splash (&state)) {
-                /* don't bother listening for udev events if we're forcing details */
+                /* don't bother listening for udev events or setting up a graphical renderer
+                 * if we're forcing details */
+                device_manager_flags |= PLY_DEVICE_MANAGER_FLAGS_SKIP_RENDERERS;
                 device_manager_flags |= PLY_DEVICE_MANAGER_FLAGS_IGNORE_UDEV;
 
                 /* don't ever delay showing the detailed splash */
@@ -2396,4 +2306,4 @@ main (int    argc,
 
         return exit_code;
 }
-/* vim: set sts=4 ts=4 sw=4 expandtab autoindent cindent cino={.5s,(0: */
+/* vim: set ts=4 ts=4 sw=4 expandtab autoindent cindent cino={.5s,(0: */
